
module Postfix::Rate::Policyd


  class Agent
    include Postfix::Rate::Policyd::Daemon

    def self.instantiate!(argv)
      new(argv).go!
    end

    attr_reader :config
    attr_reader :logger

    def initialize(argv)
      @config = Configuration.new(argv)
      configure_logger

      #configure_backend
    end


    def go!
      @exit_code = 0
      @signals_queue = []

      case config.command
        when 'setup'
          setup_backend!
        when 'start'
          start!
        when 'stop'

        when 'restart'

        when 'reload'

      end
    end

    def start!
      config.daemonize? ? daemonize { start_agent } : start_agent
      self
    end

    def setup_backend!
      setup_backend
      shutdown!
    end

    def shutdown!
      exit
    end

    private

    def start_agent
      configure_backend

      setup_signal_traps
      start_server
      idle
    end

    def configure_logger

      case config.log.to_sym
        when :file
          @logger = ::Logger.new(config.log_destination)
        when :syslog
          @logger = ::Wm::Syslog::Logger.new('postfix-rate-policyd',
                                                      facility: ::Wm::Syslog::Facility::MAIL,
                                                      level: config.log_level)
        when :remote_syslog
          @logger = ::Wm::Syslog::RemoteLogger.new('postfix-rate-policyd',
                                                      ::Wm::Syslog::Facility::MAIL,
                                                      uri: config.log_destination,
                                                      level: config.log_level)
        else
          @logger = ::Logger.new(STDOUT)
         puts "-- Logging destination is 'stdout' and process is going to daemonize." if config.daemonize?
      end
    end

    def setup_backend
      opts = config.debug? ? {loggers: [logger]} : {}

      Sequel::Model.require_valid_table = false
      Sequel::Model.db = Sequel.connect(config.database_url, opts)
      Postfix::Rate::Models.init(@config)
    end

    def configure_backend
      opts = config.debug? ? {loggers: [logger]} : {}

      Sequel::Model.require_valid_table = false
      Sequel::Model.db = Sequel.connect(config.database_url, opts)
      #Postfix::Rate::Models.init
      
      require "postfix/rate/models" 
    end

    def start_server
      Celluloid.logger = logger

      require 'postfix/rate/policyd/agent_backend'
      Postfix::Rate::Policyd::PostfixServer.start!(config.host, config.port)

    rescue Exception => e

      logger.error format_exception e
    end

    # Format an exception message
    def format_exception(exception)
      str = "#{exception.class}: #{exception.to_s}\n\t"
      if exception.backtrace
        str << exception.backtrace.join("\n\t")
      else
        str << "EMPTY BACKTRACE\n\t"
      end
    end

    #
    # sets up signal traps in a 2.0 compatible fashion
    #
    def setup_signal_traps
      %w(INT TERM USR1 USR2).each do |sig|
        trap sig do
          @signals_queue << sig
         end
       end
    end

    #
    # polls the signal queue for pending signals
    #
    def handle_signals
      while sig = @signals_queue.shift
        case sig
          when 'INT', 'TERM'
            shutdown!
        end
      end
    end

    def idle
      loop do
         handle_signals
         sleep 1
      end
    end
  end
end